@@ -19,8 +19,8 @@
 package namespaces
 
 import (
-	"github.com/containerd/containerd/namespaces"
-	"github.com/urfave/cli"
+	"github.com/containerd/containerd/v2/pkg/namespaces"
+	"github.com/urfave/cli/v2"
 )
 
 func deleteOpts(context *cli.Context) []namespaces.DeleteOpts {
