@@ -19,7 +19,7 @@ package tasks
 import (
 	gocontext "context"
 
-	"github.com/urfave/cli"
+	"github.com/urfave/cli/v2"
 )
 
 type resizer interface {
@@ -27,11 +27,11 @@ type resizer interface {
 }
 
 // Command is the cli command for managing tasks
-var Command = cli.Command{
+var Command = &cli.Command{
 	Name:    "tasks",
 	Usage:   "Manage tasks",
 	Aliases: []string{"t", "task"},
-	Subcommands: []cli.Command{
+	Subcommands: []*cli.Command{
 		attachCommand,
 		checkpointCommand,
 		deleteCommand,
